package fse.eclipse.mergehelper.detector;

import java.util.List;

import org.jtool.changerecorder.util.StringComparator;

import fse.eclipse.mergehelper.detector.merge.OperationSwapper;
import fse.eclipse.mergehelper.element.BranchFileInfo;
import fse.eclipse.mergehelper.element.BranchInfo;
import fse.eclipse.mergehelper.element.BranchRootInfo;
import fse.eclipse.mergehelper.element.ConflictInfo;
import fse.eclipse.mergehelper.element.MergeType;

public class ConflictDetector extends AbstractDetector {
    private static final String TASK_NAME = "Detect Conflict";
    private static final String ERROR_MESSAGE = "ConflictElement was Not Found";
    private static AbstractDetector instance = new ConflictDetector();

    private ConflictInfo cInfo;

    private ConflictDetector() {
    }

    public static AbstractDetector getInstance() {
        return instance;
    }

    @Override
    public void execute() {
        BranchRootInfo rootInfo = BranchRootInfo.getInstance();
        BranchInfo a_bInfo = rootInfo.getBranchInfo(MergeType.ACCEPT);
        BranchInfo j_bInfo = rootInfo.getBranchInfo(MergeType.JOIN);

        List<BranchFileInfo> abfInfos = a_bInfo.getAllBranchFileInfo();
        List<BranchFileInfo> jbfInfos = j_bInfo.getAllBranchFileInfo();

        cInfo = new ConflictInfo();
        for (BranchFileInfo abfInfo : abfInfos) {
            for (BranchFileInfo jbfInfo : jbfInfos) {
                if (abfInfo.equalsFileName(jbfInfo)) {
                    detectElement(abfInfo, jbfInfo);
                    break;
                }
            }
        }
        rootInfo.setConflictInfo(cInfo);
    }

    private void detectElement(BranchFileInfo abfInfo, BranchFileInfo jbfInfo) {
        List<String> a_elems = abfInfo.getAllElementName();
        List<String> j_elems = jbfInfo.getAllElementName();

        for (String a_elem : a_elems) {
            for (String j_elem : j_elems) {
                if (StringComparator.isSame(a_elem, j_elem)) {
                    cInfo.addConflictElements(abfInfo.getBranchJavaElement(a_elem), jbfInfo.getBranchJavaElement(j_elem));
                    break;
                }
            }
        }
    }

    @Override
    protected String getTaskName() {
        return TASK_NAME;
    }

    @Override
    protected String getErrorMessage() {
        return ERROR_MESSAGE;
    }

    @Override
    protected void nextState() {
        if (cInfo.isConflict()) {
            OperationSwapper.getInstance().detect(dialog);
        } else {
            error();
        }
    }
}
