package fse.eclipse.mergehelper.detector;

import java.io.File;
import java.io.IOException;
import java.util.ArrayList;
import java.util.List;
import java.util.Map;
import java.util.Map.Entry;
import java.util.TreeMap;

import org.apache.commons.io.FileUtils;
import org.jtool.changerecorder.history.OperationHistory;
import org.jtool.changerecorder.history.XmlConstantStrings;
import org.jtool.changerecorder.operation.IOperation;
import org.jtool.changerecorder.util.FileStream;
import org.jtool.changerecorder.util.XmlFileStream;
import org.jtool.changerepository.data.FileInfo;
import org.jtool.changerepository.operation.UnifiedOperation;
import org.w3c.dom.Document;
import org.w3c.dom.Element;
import org.w3c.dom.NodeList;

import fse.eclipse.branchrecorder.commit.history.MH_Operation2Xml;
import fse.eclipse.branchrecorder.commit.history.XmlCommitWriter;
import fse.eclipse.branchrecorder.commit.operation.CommitOperation;
import fse.eclipse.mergehelper.Activator;
import fse.eclipse.mergehelper.element.BranchInfo;
import fse.eclipse.mergehelper.element.BranchRootInfo;
import fse.eclipse.mergehelper.element.ConflictInfo;
import fse.eclipse.mergehelper.element.MergeType;
import fse.eclipse.mergehelper.util.RepositoryElementInfoUtil;

public class Debug extends AbstractDetector {
    private static final String TASK_NAME = "Debug Output";
    private static final String ERROR_MESSAGE = "";
    private static AbstractDetector instance = new Debug();
    private static int id;

    private Debug() {
    }

    public static AbstractDetector getInstance() {
        return instance;
    }

    @Override
    public void execute() {
        BranchRootInfo rootInfo = BranchRootInfo.getInstance();
        BranchInfo a_bInfo = rootInfo.getBranchInfo(MergeType.ACCEPT);
        // BranchInfo j_bInfo = rootInfo.getBranchInfo(MergeType.JOIN);

        createSnapshot(a_bInfo);
    }

    private void createSnapshot(BranchInfo bInfo) {
        List<UnifiedOperation> ops = bInfo.getProjectInfo().getOperations();
        UnifiedOperation target = null;

        id = 7886;

        for (UnifiedOperation op : ops) {
            if (op.getId() == id) {
                target = op;
                break;
            }
        }
        if (target == null) {
            return;
        }

        commit(bInfo, target);
        createHistory(bInfo, target);
    }

    private void commit(BranchInfo bInfo, UnifiedOperation target) {
        String branchName = bInfo.getName();
        List<FileInfo> fInfos = bInfo.getProjectInfo().getAllFileInfo();
        for (FileInfo fInfo : fInfos) {
            String filePath = createMergePath(fInfo, branchName);

            int idx;
            if (target.getFileInfo().equals(fInfo)) {
                idx = fInfo.indexOfOperation(target.getId());
            } else {
                idx = RepositoryElementInfoUtil.indexOfJustBeforeFileOperation(fInfo, target);
            }
            if (idx == 0) {
                continue;
            }

            String code = fInfo.getCode(idx);
            if (code != null && code.length() > 0) {
                FileStream.write(filePath, code);
            }

            String code2 = fInfo.getCode(Integer.MAX_VALUE);
            if (code2 != null && code2.length() > 0) {
                if (!org.jtool.changerecorder.util.StringComparator.isSame(code, code2)) {
                    String filePath2 = createFinalSnapshotPath(fInfo, branchName);
                    FileStream.write(filePath2, code2);
                }
            }
        }
    }

    private void createHistory(BranchInfo bInfo, UnifiedOperation target) {
        long time = target.getTime();

        Map<String, String> map = new TreeMap<String, String>();
        int idx;
        String code;

        String name = bInfo.getName();
        List<FileInfo> fInfos = bInfo.getProjectInfo().getAllFileInfo();
        for (FileInfo fInfo : fInfos) {
            List<IOperation> new_ops = new ArrayList<>();
            if (fInfo.getQualifiedName().equals(target.getFileInfo().getQualifiedName())) {
                idx = fInfo.indexOfOperation(target.getId());
                code = fInfo.getCode(idx);
                int id = target.getId();
                for (UnifiedOperation op : fInfo.getOperations()) {
                    if (op.getId() > id) {
                        new_ops.add(op.getIOperation());
                    }
                }
            } else {
                idx = RepositoryElementInfoUtil.indexOfJustBeforeFileOperation(fInfo, target);
                code = fInfo.getCode(idx);
                if (code == null) {
                    code = "";
                }

                for (UnifiedOperation op : fInfo.getOperations()) {
                    if (op.getTime() > time) {
                        new_ops.add(op.getIOperation());
                    }
                }
            }
            map.put(fInfo.getFilePath(), code);

            if (new_ops.size() > 0) {
                OperationHistory history = new OperationHistory(new_ops);
                Document doc = MH_Operation2Xml.convert(history);
                doc = validatePath(doc, name);
                write(doc, fInfo);
            }
        }

        time -= 100;
        OperationHistory commitHistory = new OperationHistory();
        for (Entry<String, String> entry : map.entrySet()) {
            String path = entry.getKey();
            code = map.get(path);
            if (code.length() > 0) {
                idx = path.indexOf(BranchHistoryCopier.BRANCH_NAME_MARK);
                int idx2 = path.indexOf("/", idx + 1);
                path = path.substring(0, idx) + path.substring(idx2);

                commitHistory.add(new CommitOperation(time, path, "", code, "null", "null"));
            }
        }

        String pname = bInfo.getProjectInfo().getName();
        pname = pname.substring(0, pname.indexOf(BranchHistoryCopier.BRANCH_NAME_MARK));

        StringBuilder sb = new StringBuilder();
        sb.append(Activator.getWorkingDirPath()).append(File.separator);
        sb.append(pname).append("_").append(id).append(File.separator);
        sb.append(time).append("_operation.xml");

        Document doc = XmlCommitWriter.generateCommitDocument(commitHistory);
        XmlFileStream.write(doc, sb.toString(), "UTF-8");
    }

    private Document validatePath(Document doc, String name) {
        // operation まで下がる
        NodeList nodeList = doc.getFirstChild().getChildNodes();
        int length = nodeList.getLength();
        int i;
        for (i = 0; i < length; i++) {
            if (nodeList.item(i).getNodeName().equals(XmlConstantStrings.OperationsElem)) {
                break;
            }
        }
        return validatePath(doc, nodeList.item(i).getChildNodes(), name);
    }

    private Document validatePath(Document doc, NodeList nodeList, String name) {
        // fileAttributeプロジェクト名の部分からブランチ名を消す
        int length = nodeList.getLength();
        for (int i = 0; i < length; i++) {
            Element e = (Element) nodeList.item(i);
            String fileAttr = e.getAttribute(XmlConstantStrings.FileAttr);
            if (!e.getNodeName().equals(XmlConstantStrings.CompoundOperationElem)) {
                if (!fileAttr.equals("null") && fileAttr.length() != 0) {
                    fileAttr = validatePath(fileAttr, name);
                    e.setAttribute(XmlConstantStrings.FileAttr, fileAttr);
                }
            } else {
                validatePath(doc, e.getChildNodes(), name);
            }
        }
        return doc;
    }

    private String validatePath(String fileAttr, String name) {
        int idx = fileAttr.indexOf(BranchHistoryCopier.BRANCH_NAME_MARK);

        StringBuilder sb = new StringBuilder();
        sb.append(fileAttr.substring(0, idx));
        sb.append(fileAttr.substring(idx + BranchHistoryCopier.BRANCH_NAME_MARK.length() + name.length()));
        return sb.toString();
    }

    private void write(Document doc, FileInfo fInfo) {
        String pname = fInfo.getProjectInfo().getName();
        pname = pname.substring(0, pname.indexOf(BranchHistoryCopier.BRANCH_NAME_MARK));

        String name = fInfo.getName();
        name = name.substring(0, name.indexOf(".java"));

        StringBuilder sb = new StringBuilder();
        sb.append(Activator.getWorkingDirPath()).append(File.separator);
        sb.append(pname).append("_").append(id).append(File.separator);
        sb.append(name).append("_operation.xml");

        XmlFileStream.write(doc, sb.toString(), "UTF-8");
    }

    private static String createMergePath(FileInfo fInfo, String branchName) {
        String filePath = fInfo.getFilePath();
        int idx = filePath.indexOf(BranchHistoryCopier.BRANCH_NAME_MARK);
        int idx2 = idx + BranchHistoryCopier.BRANCH_NAME_MARK.length() + branchName.length();

        StringBuilder sb = new StringBuilder(Activator.getWorkingDirPath());
        sb.append(filePath.substring(0, idx)).append("_").append(id);
        sb.append(filePath.substring(idx2));
        return sb.toString();
    }

    private static String createFinalSnapshotPath(FileInfo fInfo, String branchName) {
        String filePath = fInfo.getFilePath();
        int idx = filePath.indexOf(BranchHistoryCopier.BRANCH_NAME_MARK);
        int idx2 = idx + BranchHistoryCopier.BRANCH_NAME_MARK.length() + branchName.length();
        int idx3 = filePath.indexOf(".java");

        StringBuilder sb = new StringBuilder(Activator.getWorkingDirPath());
        sb.append(filePath.substring(0, idx)).append("_").append(id);
        sb.append(filePath.substring(idx2, idx3));
        sb.append("-final.java");
        return sb.toString();
    }

    public static void write(String path, String contents, boolean isAppend) {
        try {
            FileUtils.forceMkdir(new File(path).getParentFile());
        } catch (IOException e1) {
            e1.printStackTrace();
        }

        File file = new File(path);
        try {
            FileUtils.writeStringToFile(file, contents, isAppend);
        } catch (final IOException e) {
            System.err.println(e.getMessage());
        }
    }

    @Override
    protected String getTaskName() {
        return TASK_NAME;
    }

    @Override
    protected String getErrorMessage() {
        return ERROR_MESSAGE;
    }

    @Override
    protected void nextState() {
        BranchRootInfo.getInstance().setConflictInfo(new ConflictInfo());
        finish();
    }

    public static String getAMergedCode(FileInfo fInfo, String branchName) {
        String filePath = createMergePath(fInfo, branchName);
        String code = FileStream.read(filePath);
        return code;
    }

    public class MergeIndexHolder {
        private String name;
        private int idx;

        private MergeIndexHolder(FileInfo fInfo, int idx) {
            name = fInfo.getQualifiedName();
            this.idx = idx;
        }

        public int getIndex() {
            return idx;
        }

        public boolean equalsFileInfo(FileInfo fInfo) {
            return name.equals(fInfo.getQualifiedName());
        }
    }
}