package fse.eclipse.mergehelper.detector;

import java.io.File;
import java.io.IOException;
import java.util.ArrayList;
import java.util.List;

import org.apache.commons.io.FileUtils;
import org.eclipse.core.resources.ResourcesPlugin;
import org.jtool.changerecorder.history.OperationHistory;
import org.jtool.changerecorder.operation.FileOperation;
import org.jtool.changerecorder.operation.IOperation;
import org.jtool.changerecorder.util.XmlFileStream;
import org.w3c.dom.Document;

import fse.eclipse.branchrecorder.commit.history.MH_Operation2Xml;
import fse.eclipse.branchrecorder.commit.history.MH_Xml2Operation;
import fse.eclipse.branchrecorder.commit.operation.CommitOperation;
import fse.eclipse.mergehelper.Activator;
import fse.eclipse.mergehelper.element.BranchRootInfo;
import fse.eclipse.mergehelper.element.MergeType;
import fse.eclipse.mergehelper.util.FileReader;

public class HistoryNormalizer extends AbstractDetector {
    private static final String TASK_NAME = "Validate History";
    private static final String ERROR_MESSAGE = "NULL";
    private static AbstractDetector instance = new HistoryNormalizer();

    private HistoryNormalizer() {
    }

    public static AbstractDetector getInstance() {
        return instance;
    }

    @Override
    protected void execute() {
        DetectOption option = DetectOption.getInstance();
        if (option.isSlice() || option.isSwap() || option.isAMerge()) {
            BranchRootInfo rootInfo = BranchRootInfo.getInstance();

            MergeType type = MergeType.ACCEPT;
            String branchName = rootInfo.getBranchInfo(type).getName();
            StringBuilder sb = new StringBuilder();
            sb.append(Activator.getWorkingDirPath()).append(File.separator);
            sb.append(branchName);

            validateOperations(sb.toString(), type);

            type = MergeType.JOIN;
            int length = sb.length();
            sb.delete(length - branchName.length(), length);
            sb.append(rootInfo.getBranchInfo(type).getName());

            validateOperations(sb.toString(), type);
        }
    }

    private void validateOperations(String historyPath, MergeType type) {
        List<IOperation> ops = new ArrayList<>();
        List<File> files = FileReader.getAllXmlFiles(historyPath);
        for (File file : files) {
            Document doc = XmlFileStream.read(file.getAbsolutePath());
            OperationHistory history = MH_Xml2Operation.convert(doc);
            ops.addAll(history.getOperations());
        }

        OperationHistory history = new OperationHistory(ops);
        history.sort();

        List<String> addedFiles = new ArrayList<>();
        List<IOperation> new_ops = new ArrayList<>();

        List<IOperation> historyOps = history.getOperations();
        int i = 0;
        for (; i < history.size(); i++) {
            IOperation op = historyOps.get(i);
            if (op instanceof CommitOperation) {
                CommitOperation cop = (CommitOperation) op;
                String path = cop.getFilePath();
                if (!addedFiles.contains(op)) {
                    addedFiles.add(path);
                    new_ops.add(op);
                }
            } else {
                break;
            }
        }

        for (; i < history.size(); i++) {
            IOperation op = historyOps.get(i);
            if (op.getOperationType() == IOperation.Type.COMPOUND) {
                new_ops.add(op);
            } else if (op.getOperationType() == IOperation.Type.NORMAL || op.getOperationType() == IOperation.Type.COPY) {
                new_ops.add(op);
            } else if (op.getOperationType() == IOperation.Type.FILE) {
                FileOperation fop = (FileOperation) op;
                FileOperation.Type ftype = fop.getActionType();
                if (ftype == FileOperation.Type.NEW) {
                    new_ops.add(op);
                } else if (ftype == FileOperation.Type.OPEN) {
                    String path = fop.getFilePath();
                    if (!addedFiles.contains(path)) {
                        addedFiles.add(path);
                        new_ops.add(op);
                    }
                } else if (ftype == FileOperation.Type.DELETE) {
                    new_ops.add(op);

                    String path = fop.getFilePath();
                    if (addedFiles.contains(path)) {
                        addedFiles.remove(path);
                    }
                }
//            } else if (op instanceof CommitOperation) {
            }
        }

        try {
            FileUtils.cleanDirectory(new File(historyPath));
        } catch (IOException e) {
            e.printStackTrace();
        }

        StringBuilder sb = new StringBuilder();
        sb.append(historyPath).append(File.separator);
        sb.append("validate.xml");

        Document doc = MH_Operation2Xml.convert(new OperationHistory(new_ops));
        XmlFileStream.write(doc, sb.toString(), ResourcesPlugin.getEncoding());
    }

    @Override
    protected String getTaskName() {
        return TASK_NAME;
    }

    @Override
    protected String getErrorMessage() {
        return ERROR_MESSAGE;
    }

    @Override
    protected void nextState() {
        ChangeRepositoryCreator.getInstance().detect(dialog);
    }
}
