package fse.eclipse.mergehelper.element;

import java.util.ArrayList;
import java.util.Collections;
import java.util.Comparator;
import java.util.List;

import org.jtool.changerecorder.util.StringComparator;
import org.jtool.changerepository.operation.UnifiedOperation;
import org.jtool.changerepository.parser.OpJavaElement;

public class BranchJavaElement {
    private final BranchFileInfo bfInfo;
    private final OpJavaElement elem;
    private final List<UnifiedOperation> ops = new ArrayList<>();

    BranchJavaElement(BranchFileInfo bfInfo, OpJavaElement elem) {
        this.bfInfo = bfInfo;
        this.elem = elem;
    }

    public BranchFileInfo getBranchFileInfo() {
        return bfInfo;
    }

    public OpJavaElement getElement() {
        return elem;
    }

    public String getName() {
        return elem.getName();
    }

    public String getSimpleName() {
        return elem.getSimpleName();
    }

    public String getFullName() {
        return elem.getFullName();
    }

    public String getPackageName() {
        return elem.getFileInfo().getPackageInfo().getName();
    }

    public String getFileName() {
        return elem.getFileInfo().getName();
    }

    public List<UnifiedOperation> getOperations() {
        return ops;
    }

    public void addOperation(UnifiedOperation op) {
        ops.add(op);
    }

    public boolean isEdited() {
        return ops.size() > 0;
    }

    public boolean contains(UnifiedOperation op) {
        return ops.contains(op);
    }

    public boolean equalsFileElement(BranchJavaElement elem) {
        if (!StringComparator.isSame(this.getPackageName(), elem.getPackageName())) {
            return false;
        }

        if (!StringComparator.isSame(this.getFileName(), elem.getFileName())) {
            return false;
        }

        if (!StringComparator.isSame(this.getName(), elem.getName())) {
            return false;
        }
        return true;
    }

    public static void sort(List<BranchJavaElement> elems) {
        Collections.sort(elems, new Comparator<BranchJavaElement>() {
            @Override
            public int compare(BranchJavaElement e1, BranchJavaElement e2) {
                return e1.getSimpleName().compareTo(e2.getSimpleName());
            }
        });
    }

    @Override
    public String toString() {
        StringBuilder sb = new StringBuilder();
        sb.append(getName()).append(" op[ ");
        for (UnifiedOperation op : getOperations()) {
            sb.append(op.getId()).append(" ");
        }
        sb.append("]");
        return sb.toString();
    }
}
