package fse.eclipse.mergehelper.element;

import java.math.BigDecimal;
import java.math.RoundingMode;
import java.util.ArrayList;
import java.util.Comparator;
import java.util.List;

import org.jtool.changerepository.operation.UnifiedOperation;

public class MergePoint {
    private List<Point> points = new ArrayList<>();
    private int refIdx = -1;

    public MergePoint() {
    }

    public void sort() {
        points.sort(new Comparator<Point>() {
            @Override
            public int compare(Point p1, Point p2) {
                return p2.idx_per.compareTo(p1.idx_per);
            }
        });
    }

    public void setReferenceIndex(int refIdx) {
        this.refIdx = refIdx;
    }

    public int size() {
        return points.size();
    }

    public String getReferenceIndexPercent(MergeType type) {
        return calcIndexPercentage(getReferenceOperation(type)).toString();
    }

    public UnifiedOperation getReferenceOperation(MergeType type) {
        return getOperation(refIdx, type);
    }

    public UnifiedOperation getOperation(int idx, MergeType type) {
        if (idx < 0 || size() <= idx) {
            return null;
        }
        if (MergeType.isAccept(type)) {
            return points.get(idx).a_op;
        } else {
            return points.get(idx).j_op;
        }
    }

    public void addPoint(UnifiedOperation a_op, UnifiedOperation j_op) {
        points.add(new Point(a_op, j_op));
    }

    private BigDecimal calcIndexPercentage(UnifiedOperation op) {
        BigDecimal idx = BigDecimal.valueOf(op.indexOfProjectInfo() + 1);
        BigDecimal total = BigDecimal.valueOf(op.getProjectInfo().getOperationNumber());
        return idx.divide(total, 4, RoundingMode.HALF_UP);
    }

    private class Point {
        UnifiedOperation a_op;
        UnifiedOperation j_op;
        BigDecimal idx_per;

        Point(UnifiedOperation a_op, UnifiedOperation j_op) {
            this.a_op = a_op;
            this.j_op = j_op;
            idx_per = calcIndexPercentage(a_op).add(calcIndexPercentage(j_op));
        }
    }
}
