package fse.eclipse.mergehelper.ui.dialog;

import java.lang.reflect.InvocationTargetException;

import org.eclipse.core.runtime.IProgressMonitor;
import org.eclipse.jface.dialogs.ProgressMonitorDialog;
import org.eclipse.jface.operation.IRunnableWithProgress;
import org.eclipse.swt.widgets.Shell;

import fse.eclipse.mergehelper.detector.AbstractDetector;
import fse.eclipse.mergehelper.detector.InitWorkingDirectory;
import fse.eclipse.mergehelper.ui.MH_PackageExplorerView;

public class ConflictDetectingDialog implements IRunnableWithProgress {
    private static final String TITLE = "Please Wait ...";

    private ProgressMonitorDialog pDialog;
    private AbstractDetector instance;

    public ConflictDetectingDialog(ProgressMonitorDialog pDialog) {
        this.pDialog = pDialog;
        setFirstDetector(InitWorkingDirectory.getInstance());
    }

    public void setFirstDetector(AbstractDetector instance) {
        this.instance = instance;
    }

    public void open() {
        pDialog.open();

        Shell dialog = pDialog.getShell();
        dialog.setText(TITLE);
        dialog.update();

        try {
            pDialog.run(false, false, this);
        } catch (InvocationTargetException | InterruptedException e) {
            e.printStackTrace();
        }
    }

    @Override
    public void run(IProgressMonitor monitor) throws InvocationTargetException, InterruptedException {
        monitor.beginTask("", AbstractDetector.STATE_NUMBER);
        monitor.setCanceled(false);

        instance.detect(this);
    }

    public void setTaskName(String name) {
        pDialog.getProgressMonitor().setTaskName(name);
    }

    public void setSubTaskName(String name) {
        pDialog.getProgressMonitor().subTask(name);
    }

    public void worked() {
        pDialog.getProgressMonitor().worked(1);
    }

    public void detectSuccess() {
        pDialog.close();
        pDialog.getShell().dispose();
        System.out.println("detect finish");

        IResultDialog nextDialog = new DetectionResultDialog(new Shell());
        nextDialog.result();

        MH_PackageExplorerView.getInstance().show(nextDialog);
    }

    public void detectFailed(String errorMessage) {
        pDialog.close();
        pDialog.getShell().dispose();
        System.err.println("detect finish (failed)");

        IResultDialog nextDialog = new DetectionErrorDialog(new Shell(), errorMessage);
        nextDialog.result();

        MH_PackageExplorerView.getInstance().show(nextDialog);
    }
}
